import logging
import schedule
import time

from googleapiclient.errors import HttpError
from . import message

logger = logging.getLogger(__name__)


class Dispatcher:
    def __init__(self, service):
        self.service = service
        self.message_handler = message.MessageHandler(self.service)

    def watch_inbox(self, pubsub_request):
        # Call `watch()` on inbox once per day
        schedule.every().day.do(
            self._watch_inbox_helper,
            request=pubsub_request)

        schedule.run_all()
        while True:
            schedule.run_pending()
            time.sleep(1)

    def _watch_inbox_helper(self, request):
        self.service.users().watch(
            userId='me',
            body=request).execute()

    def check_inbox(self, query):
        try:
            received_messages = self._list_messages(query)
            processed_payload = self.message_handler.process_messages(received_messages)

            return processed_payload
        except Exception as error:
            logger.error('Cannot get mail: {}'.format(error))

    def _list_messages(self, query):
        try:
            response = self.service.users().messages().list(
                userId='me',
                q=query).execute()

            messages = []
            if 'messages' in response:
                messages.extend(response['messages'])

            while 'nextPageToken' in response:
                page_token = response['nextPageToken']
                response = self.service.users().messages().list(
                    userId='me',
                    q=query,
                    pageToken=page_token).execute()
                messages.extend(response['messages'])

            return messages
        except HttpError as error:
            print('list_messages error: {}'.format(error))



from google.oauth2 import service_account
import googleapiclient.discovery


SCOPES = [
    'https://www.googleapis.com/auth/gmail.modify'
]


class Service:
    def __init__(self, delegate_email, creds_file, creds_dir):
        creds_path = '{}/{}'.format(creds_dir, creds_file)

        credentials = service_account.Credentials.from_service_account_file(
            creds_path, scopes=SCOPES).with_subject(delegate_email)

        try:
            self._build_service(credentials)
        # Try again incase http://www.googleapis.com/ cannot be reached from within docker
        except Exception:
            self._build_service(credentials)

    def use(self):
        return self.authorized_service

    def _build_service(self, credentials):
        self.authorized_service = googleapiclient.discovery.build(
            'gmail', 'v1', credentials=credentials, cache_discovery=False)



import setuptools

with open('README.md', 'r') as fh:
    long_description = fh.read()

setuptools.setup(
    name='gmaild',
    version='0.0.11',
    author='Parker Duckworth',
    description='Gmail Daemon',
    long_description=long_description,
    long_description_content_type='text/markdown',
    packages=setuptools.find_packages(),
    license='Proprietary',
    install_requires=[
        'google-api-python-client',
        'google-auth-httplib2',
        'google-auth-oauthlib',
        'schedule'
    ],
    python_requires='>=3.6',
    include_package_data=True,
    zip_safe=False
)



import threading

from . import auth, dispatch


class Courier:
    def __init__(self, delegate_email, creds_file, creds_dir='.'):
        authorized_service = auth.Service(delegate_email, creds_file, creds_dir)
        self.dispatcher = dispatch.Dispatcher(authorized_service.use())

    def run(self, label_ids, filter_action, topic_name):
        # TODO: validate input
        pubsub_request = {
            'labelIds': label_ids,
            'labelFilterAction': filter_action,
            'topicName': topic_name
        }

        # Calls watch() on inbox once per day as recommended by google api docs
        watcher_thread = threading.Thread(
            target=self.dispatcher.watch_inbox,
            args=(pubsub_request,),
            daemon=True)

        watcher_thread.start()

    # Returns all messages that match the query string
    def deliver_mail(self, query):
        return self.dispatcher.check_inbox(query)
